--[[ Roster module to Handle the Raid Roster    ]]
--[[ tab widths set to 4 for optimal formatting ]]

local _, TriageTable = ...; 

local Triage = TriageTable.Triage
Triage.Dialog = Triage:NewModule("Dialog", "AceEvent-3.0", "AceBucket-3.0")
local Dialog = Triage.Dialog

local L = LibStub("AceLocale-3.0"):GetLocale("Triage");


-- The various state tables.
Dialog.heal = { }
Dialog.heal.unassigned = { }
Dialog.heal.assigned = { }
Dialog.dps = { }
Dialog.dps.unassigned = { }
Dialog.dps.assigned = { }
Dialog.tank = { }
Dialog.tank.unassigned = { }
Dialog.tank.assigned = { }
Dialog.other = { }
Dialog.other.unassigned = { }
Dialog.other.assigned = { }

Dialog.role = { }
Dialog.active_tab = "HEALER"

Dialog.unassigned_slot_list = { }
Dialog.assignment_slot_list = { }

Dialog.alpha_raid = 0.9
Dialog.alpha_waiting = 0.3



function Dialog:Debug( str, ... )
	Triage:Debug("|c933f9fffDialog|r - "..str , ... )
end



function Dialog:OnInitialize() 
	self:Debug( "OnInitialize" )
end



function Dialog:OnEnable()
	self:Debug( "OnEnable" )

	if not self.frame then
		self:CreateDialog()
	end

	self:RegisterMessage( "Triage_RosterUpdated", "RosterUpdated" )

	if oRA3 then
		self:Debug( "Registering for oRA3 Tanks" )
		oRA3.RegisterCallback( self, "OnTanksUpdated", "ORA3TanksUpdated" )
		self:ORA3TanksUpdated( "OnTanksUpdated", oRA3:GetSortedTanks() )
	end

	self:Debug( "OnEnable complete" )
end



function Dialog:OnLoad()
	self:Debug( "OnLoad" )
end



function Dialog:CreateDialog()
	self:Debug( "CreateDialog" )

	-- Sizes/counts of various elements
	self.available_size = 25
	self.group_count = 8
	self.group_size = 5

	-- Localize the XML items
	_G["TriageDialogAvailableLabel"]:SetText(L["Unassigned"])
	for i = 1, self.available_size do
		local key = ( "TriageDialogAvailableSlot%dLabel" ):format( i )
		local value = "Empty"
		_G[key]:SetText( L[value] )
	end

	for i = 1, self.group_count do
		local key = ( "TriageDialogAssignmentGroup%dLabel" ):format( i )
		local value = ( "Assignment %d" ):format( i )
		_G[key]:SetText( L[value] )
		for k = 1, self.group_size do
			key = ( "TriageDialogAssignmentGroup%dSlot%dLabel" ):format( i, k )
			value = "Empty"
			_G[key]:SetText( L[value] )
		end
	end


	-- Create assignment groups tables
	for i = 1, self.group_count do
		self.heal.assigned[ i ] = { }
		self.dps.assigned[ i ] = { }
		self.tank.assigned[ i ] = { }
		self.other.assigned[ i ] = { }

		-- Save the destination slot names to the assignment group slot list	
		for k = 1, self.group_size do
			local key = ( "TriageDialogAssignmentGroup%dSlot%d" ):format( i, k )
			table.insert( self.assignment_slot_list, key )
		end
	end

	-- Save the available slot names to the unassigned slot list	
	for i = 1, self.available_size do
		local key = ( "TriageDialogAvailableSlot%d" ):format( i )
		table.insert( self.unassigned_slot_list, key )
	end

	-- Create the buttons
	local frame_level = TriageDialog:GetFrameLevel()
	self.available_buttons = { }
	for i = 1, self.available_size do
		local name = ( "TriagePlayerButton%d" ):format( i )
		local button = CreateFrame( "Button", name, TriageDialog, 
								"TriageButtonTemplate" )
		button:ClearAllPoints()
		button:SetFrameLevel( frame_level + 5 )
		button:SetMovable( true )
		button:RegisterForDrag( "LeftButton" )
		button:EnableMouse( true )
		button:SetClampedToScreen( true )
		button:Hide()
		self.available_buttons[ i ] = button
		self.next_available_button = i
	end

	self.frame = TriageDialog
	self:Debug( "CreateDialog complete" )
end



function Dialog:RosterUpdated( event )
	self:Debug( "RosterUpdated" )
	local raid_size, raid_roster, waiting_roster, class_table, role_table =
		Triage.Roster:GetRaidRoster()

	if Triage.debug then
		raid_list = ( "Raid size: %d -- Inside:" ):format( raid_size )
		for i = 1, #raid_roster do
    		raid_list = raid_list.." "..raid_roster[i]
		end
		raid_list = raid_list.." -- Outside:"
		for i = 1, #waiting_roster do
    		raid_list = raid_list.." "..waiting_roster[i]
		end
		self:Debug( raid_list )
	end

	for i = 1, #raid_roster do
		local name = raid_roster[ i ]
		local role = role_table[ name ]
		self:PopulateTables( name, role )
	end

	for i = 1, #waiting_roster do
		local name = waiting_roster[ i ]
		local role = role_table[ name ]
		self:PopulateTables( name, role )
	end

	self:BuildActiveTab( raid_roster, waiting_roster, class_table )
end



function Dialog:PopulateTables( name, role )

	local unassigned, assigned = self:GetActiveTables( role )

	local previous_role = self.role[ name ]
	if previous_role then
		if previous_role ~= role then
			local prev_unassigned, prev_assigned = 
				self:GetActiveTables( previous_role )
			self:RemovePlayerFromTables( prev_unassigned, prev_assigned, name )
		end
		self.role[ name ] = role
	end

	if role ~= "NONE" then
		local found = false
		if tContains( unassigned, name ) then
			self:Debug( name .. " found in unassigned for " .. role )
			found = true
		else
			for i, v in ipairs( assigned ) do
				if tContains( v, name ) then
					self:Debug( name .. " found in assigned for " .. role )
					found = true
					break
				end
			end
		end

		if not found then
			if not tContains( unassigned, name ) then
				table.insert( unassigned, name )
				self:Debug( name .. " added to unassigned for " .. role )
			end
		end
	end
end



function Dialog:BuildActiveTab( raid_roster, waiting_roster, class_table )
	self:Debug( "Building active tab: " .. self.active_tab )

	local unassigned, assigned = self:GetActiveTables( self.active_tab )	

	self.next_available_button = 25
	for i = 1, #self.available_buttons do
		local button = self.available_buttons[ i ]
		button:Hide()
	end

	local raid_slot = 1
	local waiting_slot = 25
	for i = 1, #unassigned do
		local name = unassigned[ i ]
		self:Debug( "** Unassigned player " .. name )

		local in_raid = false
		local waiting = false
		if tContains( raid_roster, name ) then
			in_raid = true
		elseif tContains( waiting_roster, name ) then
			waiting = true
		end

		if in_raid or waiting then
			local slot
			if in_raid then
				slot = ( "TriageDialogAvailableSlot%d" ):format( raid_slot )
				raid_slot = raid_slot + 1
			else
				slot = ( "TriageDialogAvailableSlot%d" ):format( waiting_slot )
				waiting_slot = waiting_slot - 1
			end

			local class = strupper( class_table[ name ] )
			self:Debug( name .. " is " .. class ) 

			self:PrepareButton( slot, name, class, in_raid )
		else
			self:Debug( name .." not found in raid or waiting rosters" )
			self:RemovePlayerFromTables( unassigned, assigned, name )
		end
	end

	for i = 1, #assigned do 
		local group = assigned[ i ]
		for k = 1, #group do
			local name = group[ k ]
			self:Debug( ( "Group %d, Name %d is %s" ):format( i, k, name ) )

			local in_raid = false
			local waiting = false
			if tContains( raid_roster, name ) then
				in_raid = true
			elseif tContains( waiting_roster, name ) then
				waiting = true
			end

			if in_raid or waiting then
				local slot = ( "TriageDialogAssignmentGroup%dSlot%d" ):format( i, k )
				local class = strupper( class_table[ name ] )
				self:Debug( name .. " is " .. class ) 

				self:PrepareButton( slot, name, class, in_raid )
			else
				self:Debug( name .." not found in raid or waiting rosters" )
				self:RemovePlayerFromTables( unassigned, assigned, name )
			end
		end
	end

end



function Dialog:PrepareButton( slot, name, class, in_raid )
	local button = self.available_buttons[ self.next_available_button ]
	self.next_available_button = self.next_available_button - 1
	button:ClearAllPoints()
	button:SetPoint( "TOPLEFT", slot, "TOPLEFT" )

	local label = _G[button:GetName().."Label"]
	label:SetText( name )

	local color = RAID_CLASS_COLORS[class]
	local button_color = _G[button:GetName().."Color"]
	if in_raid then
		button_color:SetTexture( color.r, color.g, color.b, self.alpha_raid )
	else
		button_color:SetTexture( color.r, color.g, color.b, self.alpha_waiting )
	end

	button:Show()
end



function Dialog:GetActiveTables( role )
	-- Build the active tab based on the state
	if role == "HEALER" then
		unassigned = self.heal.unassigned
		assigned = self.heal.assigned
	elseif role == "DAMAGER" then
		unassigned = self.dps.unassigned
		assigned = self.dps.assigned
	elseif role == "TANK" then
		unassigned = self.tank.unassigned
		assigned = self.tank.assigned
	end

	return unassigned, assigned
end



function Dialog:ORA3TanksUpdated( event, tanks )
	self.ora3_tanks = tanks
	local tank_list = "oRA3 Tanks:"
	for i = 1, #tanks do
		tank_list = tank_list.." "..tanks[ i ]
	end
	Dialog:Debug( tank_list )
end



-- Deprecated, kept for now for reference.  TODO: delete
function Dialog:CreateMainDialog()
	self:Debug( "CreateMainDialog" )

	local font = media and media:Fetch("font", self.db.profile.font) or STANDARD_TEXT_FONT

	local frame = CreateFrame( "Frame", "TriageDialog", UIParent )

	frame:SetWidth( 480 )
	frame:SetHeight( 600 )

	frame:SetFrameStrata( "DIALOG")
	frame:SetToplevel( true )
	frame:EnableMouse( true )
	frame:SetClampedToScreen( true )

	frame:SetMovable( true )
	frame:SetPoint( "TOP", UIParent, "TOP", 0, -50 )

	frame:SetBackdrop{
		bgFile="Interface\\DialogFrame\\UI-DialogBox-Background" ,
		edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border",
		tile = true,
		insets = {left = 11, right = 12, top = 12, bottom = 11},
		tileSize = 32, edgeSize = 32,
	}

	frame.texture = frame:CreateTexture( nil, "BORDER" )
	frame.texture:SetPoint( "TOPLEFT", frame, "TOPLEFT", 4, -4 )
	frame.texture:SetPoint( "BOTTOMRIGHT", frame, "BOTTOMRIGHT", -4, 4 )
	frame.texture:SetBlendMode( "ADD" )
	frame.texture:SetGradientAlpha( "VERTICAL", .1, .1, .1, 0, .2, .2, .2, 0.5 )
	frame:EnableMouse( true )
	
	local titleRegion = frame:CreateTitleRegion()
	titleRegion:SetPoint( "TOP", frame, "TOP", 0, 22 )
	titleRegion:SetPoint( "LEFT", frame, "TOP", -50, 0 )
	titleRegion:SetPoint( "RIGHT", frame, "TOP", 50, 0 )
	titleRegion:SetPoint( "BOTTOM", frame, "TOP", 0, -16 )

	local header = frame:CreateTexture( nil, "ARTWORK" )
	header:SetTexture( "Interface\\DialogFrame\\UI-DialogBox-Header" )
	header:SetWidth( 256 )
	header:SetHeight( 64 )
	header:SetPoint( "TOP", 0, 12 )

	local title = frame:CreateFontString( "ARTWORK" )
	title:SetFontObject( "GameFontNormal" )
	title:SetPoint( "TOP", header, "TOP", 0, -14 )
	title:SetText( L["Triage"] )

	local closeButton = CreateFrame( "Button", "CloseDialogButton",
		frame, "UIPanelCloseButton")
	closeButton:SetPoint( "TOPRIGHT", 0, 0)
	closeButton:SetScript( "OnClick", function( self ) Dialog:HideDialog() end )

	frame:SetScript( "OnShow", function() PlaySound( "igMainMenuOption" ) end )
	frame:SetScript( "OnHide", function() PlaySound( "gsTitleOptionExit" ) end )

	frame:Hide()
	tinsert( UISpecialFrames,"TriageDialog" );
	return frame
end



function Dialog:CreateAvailable( parent )
	self:Debug( "CreateAvailable" )

	local frame = CreateFrame( "Frame", "TriageAvailable", parent )

	frame:SetWidth( 100 )
	frame:SetHeight( 261 )

	frame:SetPoint( "TOPLEFT", parent, "TOPLEFT", 14, -45 )

	frame:SetBackdrop{
		bgFile="Interface\\DialogFrame\\UI-DialogBox-Background" ,
--		edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border",
		tile = true,
		insets = {left = 1, right = 1, top = 1, bottom = 1},
		tileSize = 32, edgeSize = 32,
	}

	frame.texture = frame:CreateTexture( nil, "BORDER" )
	frame.texture:SetPoint( "TOPLEFT", frame, "TOPLEFT", 4, -4 )
	frame.texture:SetPoint( "BOTTOMRIGHT", frame, "BOTTOMRIGHT", -4, 4 )
	frame.texture:SetBlendMode( "DISABLE" )
--	frame.texture:SetGradientAlpha( "VERTICAL", .1, .1, .1, 0, .2, .2, .2, 0.5 )
	frame:EnableMouse( true )

	frame.slots = { }
	for index, slotname in ipairs( self.available ) do
		local slot = self:CreateAvailableSlot( frame, index, slotname )
		frame.slots[ index ] = slot
	end

	parent.availableFrame = frame
end




function Dialog:Dialog()
	self:Debug( "Dialog" )
	if self.frame then
		if self.frame:IsShown() then
			self:HideDialog()
		else
			self:ShowDialog()
		end
	end
end



function Dialog:ShowDialog()
	self:Debug( "ShowDialog" )
	if Triage.inCombat then
		self:Debug( "  in combat, refusing to display" );
		return
	end
	if self.frame then
		self.frame:Show()
	else 
		 TriageDialog:Show()
	end
	
end



function Dialog:HideDialog()
	self:Debug( "HideDialog" )
	if self.frame then
		self.frame:Hide()
	end
end



function Dialog:PlayerButtonOnLoad( button )
	self:Debug( "PlayerButtonOnLoad: "..button:GetName() )
end



function Dialog:PlayerButtonOnClick( button )
	self:Debug( "PlayerButtonOnClick: "..button:GetName() )
end



function Dialog:PlayerButtonOnDragStart( button )
	self:Debug( "PlayerButtonOnDragStart: "..button:GetName() )
	button:ClearAllPoints()

	local moving_level = button:GetParent():GetFrameLevel() + 40;
	button:SetFrameLevel( moving_level )
	button:StartMoving()
end




function Dialog:PlayerButtonOnDragStop( button )
	self:Debug( "PlayerButtonOnDragStop: "..button:GetName() )

	local stopped_level = button:GetParent():GetFrameLevel() + 5
	button:SetFrameLevel( stopped_level )
	button:StopMovingOrSizing()

	local unassigned, assigned = self:GetActiveTables( self.active_tab )

	local found = false

	-- check the assignment groups first
	for i, slot_name in ipairs( self.assignment_slot_list ) do
		local slot = _G[slot_name]
		if MouseIsOver( slot ) then
			local player = _G[button:GetName() .. "Label"]:GetText()
			self:Debug( "Dropping " .. button:GetName() .. " - " .. player .. " over " .. slot_name )

			-- Look for something in the format of ".*Group%dSlot%d"
			local _, _, group_str, slot_str = 
				string.find( slot_name, ".+(%d+).+(%d+)" )
			--self:Debug( ( "Group %d, Slot %d" ):format( group_num, slot_num ) )
			local group_num = tonumber( group_str )
			self:RemovePlayerFromTables( unassigned, assigned, player )

			local destination = assigned[ group_num ]
			table.insert( destination, player )
			table.sort( destination )
			found = true
		end
	end

	-- check the unassigned list if not found yet
	if not found then
		for i, slot_name in ipairs( self.unassigned_slot_list ) do
			local slot = _G[slot_name]
			if MouseIsOver( slot ) then
				local player = _G[button:GetName() .. "Label"]:GetText()
				self:Debug( "Dropping " .. button:GetName() .. " - " .. player .. " over " .. slot_name )

				-- Look for something in the format of ".*Slot%d"
				local _, _, slot_num = string.find( slot_name, ".+(%d+)" )
				--self:Debug( ( "AvailableSlot %d" ):format( slot_num ) )

				self:RemovePlayerFromTables( unassigned, assigned, player )
				table.insert( unassigned, player )
				table.sort( unassigned )
				found = true
			end
		end
	end

	self:RosterUpdated()
end



function Dialog:RemovePlayerFromTables( unassigned, assigned, name )
	self:Debug( "RemovePlayerFromTables: "..name )	

	local limit = #unassigned
	for i = 1, limit do
		if unassigned[ i ] == name then
			unassigned[ i ] = unassigned[ limit ]
			unassigned[ limit ] = nil
			return
		end
	end

	for group_num = 1, #assigned do
		local group = assigned[group_num]
		local limit = #group
		for i = 1, limit do
			if group[ i ] == name then
				group[ i ] = group[ limit ]
				group[ limit ] = nil
				return
			end
		end
	end
end



function Dialog:ReportHealers()
	self:Debug("ReportHealers")
	local unassigned, assigned = self:GetActiveTables( "HEALER" )
	self:BuildReport( unassigned, assigned, "Healing" )
end



function Dialog:ReportDamagers()
	self:Debug("ReportDamagers")
	local unassigned, assigned = self:GetActiveTables( "DAMAGER" )
	self:BuildReport( unassigned, assigned, "DPS" )
end



function Dialog:ReportTanks()
	self:Debug("ReportTanks")
	local unassigned, assigned = self:GetActiveTables( "TANK" )
	self:BuildReport( unassigned, assigned, "Tanking" )
end



function Dialog:BuildReport( unassigned, assigned, role )

	local _, raid_roster, waiting_roster = Triage.Roster:GetRaidRoster()
	
	for i = 1, #assigned do
		local group = assigned[ i ]
		if #group > 0 then
			-- TODO: get actual assignment from assignment group
			local output = ( "Assignment %d:" ):format( i )
			for k = 1, #group do
				local name = group[ k ]
				if tContains( raid_roster, name ) then
					output = output .. " " .. name
					in_raid = true
				elseif tContains( waiting_roster, name ) then
					output = output .. " " .. name .. "-Out"
					waiting = true
				end
			end
			Triage:SendOutput( output )
		end
	end

	if #unassigned > 0 then
		local output = "Unassigned:"
		for i = 1, #unassigned do
			local name = unassigned[ i ]
			if tContains( raid_roster, name ) then
				output = output .. " " .. name
			end
		end
		Triage:SendOutput( output )
	end

end