--[[ Roster module to Handle the Raid Roster    ]]
--[[ tab widths set to 4 for optimal formatting ]]

local _, TriageTable = ...; 

local Triage = TriageTable.Triage
Triage.Roster = Triage:NewModule( "Roster", "AceEvent-3.0" )
local Roster = Triage.Roster

Roster.TalentQuery = LibStub:GetLibrary( "LibTalentQuery-1.0" )


-- Use active and internal as flip buffers..
-- update one, then swap with the other so changes are consistent.
local active = {
	raid_roster = { },
	waiting_roster = { },
	guid = { },
	unitid = { },
	name = { },
	class = { },
	group = { },
	role = { },
};

local internal = {
	raid_roster = { },
	waiting_roster = { },
	guid = { },
	unitid = { },
	name = { },
	class = { },
	group = { },
	role = { },
};



function Roster:Debug(str, ...)
	Triage:Debug("|c933f9fffRoster|r - "..str , ... )
end



--[[ Primary module callbacks, initialize, enable, disable ]]
function Roster:OnInitialize()
	-- Called at module load
	self:Debug("OnInitialize")

	self.TalentQuery.RegisterCallback( self, "TalentQuery_Ready" )
	self:OnEnable()
end



function Roster:OnEnable()
	-- Called when the module is enabled
	self:Debug("OnEnable")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:RegisterEvent("UNIT_NAME_UPDATE")

	self:Update()
end



function Roster:OnDisable()
	self:Debug("OnDisable")
	-- Called when the module is disabled
end



--[[ Event handlers ]]
function Roster:PLAYER_ENTERING_WORLD()
	self:Debug("PLAYER_ENTERING_WORLD")
	self:Update()
end



function Roster:RAID_ROSTER_UPDATE()
	self:Debug("RAID_ROSTER_UPDATE")
	self:Update()
end



function Roster:UNIT_NAME_UPDATE()
	self:Debug("UNIT_NAME_UPDATE")
	self:Update()
end



--[[ Update the raid member list ]]
function Roster:Update()
	-- self:Debug("Update")

	if GetNumRaidMembers() == 0 then return end

	if Triage.inCombat then
		self.update_in_combat = true
		self:Debug("Update delayed due to combat")
		return
	end

	local _, _, _, _, maxPlayers = GetInstanceInfo()

	-- If we're not actually in the instance, get the max number of players
	-- by looking at the difficulty.. 
	if maxPlayers == 0 then
		difficultyIndex = GetRaidDifficulty()
		-- self:Debug( ("Difficulty %d"):format(difficultyIndex) )
		if difficultyIndex == 1 or difficultyIndex == 3 then
			maxPlayers = 10
		else
			maxPlayers = 25
		end
	end
	self.instanceSize = maxPlayers
	self.maxGroupInRaid = maxPlayers / 5
	-- self:Debug( ("Max Players: %d, Max Groups: %d"):format(maxPlayers, self.maxGroupInRaid) )

	self:WipeInternalTables()
	for i = 1, MAX_RAID_MEMBERS do
		local unitid = ("raid%d"):format( i )
		local guid = UnitGUID( unitid )
		if guid then
			local name, rank, subgroup, level, class, fileName, zone,
				online, isDead, raid_role, isML = GetRaidRosterInfo( i )
			if name then
				internal.guid[name]   = guid
			    internal.unitid[name] = unitid
				internal.name[guid]   = name
				internal.class[name]  = class
				internal.group[name]  = subgroup
				if subgroup and subgroup <= self.maxGroupInRaid then
					table.insert( internal.raid_roster, name )
					-- self:Debug( ("%s in raid, group %d, "):format(name, subgroup)..internal.class[name]..", "..internal.role[name] )
				else
					table.insert( internal.waiting_roster, name )
					-- self:Debug( ("%s outside, group %d, "):format(name, subgroup)..internal.class[name]..", "..internal.role[name] )
				end

				local role = UnitGroupRolesAssigned( unitid )
				if role ~= "NONE" then
					internal.role[name] = role
				elseif class == "Hunter" or class == "Mage" or
					class == "Rogue" or class == "Warlock" then
					internal.role[name] = "DAMAGER"
				else
					internal.role[name] = "NONE"
					self.TalentQuery:Query( unitid )
				end
			end
		end
	end

	-- Used to try and determine if we updated or not.. if we were called,
   	-- assume something we care about changed.  It's cheaper to process the
	-- update event later than do the comparisons.

	-- We still want the tables to be sorted 'tho.. 
	table.sort( internal.raid_roster )
	table.sort( internal.waiting_roster )

	-- flip the tables.
	local temp = active
	active = internal
	internal = temp
	self:WipeInternalTables()

	if Triage.debug and false then
		raid_list = ("Raid size: %d -- Inside:"):format(maxPlayers)
		for i = 1, #active.raid_roster do
    		raid_list = raid_list.." "..active.raid_roster[i]
		end
		raid_list = raid_list.." -- Outside:"
		for i = 1, #active.waiting_roster do
    		raid_list = raid_list.." "..active.waiting_roster[i]
		end
		self:Debug(raid_list)
	end

	self:Debug("Sending RosterUpdated message")
	self:SendMessage( "Triage_RosterUpdated" )
end



function Roster:WipeInternalTables()
	table.wipe( internal.raid_roster )
	table.wipe( internal.waiting_roster )
	table.wipe(	internal.guid )
	table.wipe( internal.unitid )
	table.wipe(	internal.name )
	table.wipe(	internal.class )
	table.wipe(	internal.group )
	table.wipe(	internal.role )
end



-- Note: the keys for the rosters is an integer from 1 to the roster length
--       the keys for the class, role, and group tables is the player name.
function Roster:GetRaidRoster()
	return self.instanceSize, active.raid_roster, active.waiting_roster,
			active.class, active.role, active.group
end



function Roster:GetRaidIDs()
	return active.guid, active.unitid, active.name
end



function Roster:TalentQuery_Ready( event, name, realm, unitid )
	self:Debug( "TalentQuery_Ready for " .. name )

	if active.role[name] ~= "NONE" then
		self:Debug( "Role assigned, returning" )
		return
	end

	local isnotplayer = not UnitIsUnit( unitid, "player" )
	local group = GetActiveTalentGroup( isnotplayer )

	local highest_tree = "NONE"
	local max_points = 0

	for tab = 1, GetNumTalentTabs( isnotplayer ) do
		local _, treename, _, _, pointsspent = 
			GetTalentTabInfo( tab, isnotplayer, nil, group )

			--self:Debug( ("%s, %s, %d"):format(name, treename , pointsspent ) )
			if pointsspent > max_points then
				--self:Debug( ("assigning %s %d %d"):format( treename, pointsspent, max_points ) )
				highest_tree = treename
				max_points = pointsspent
			end
	end

	if active.role[name] == "NONE" then
		if highest_tree == "Restoration" or highest_tree == "Holy" or
			highest_tree == "Discipline" then
			active.role[name] = "HEALER"
		elseif highest_tree == "Protection" or highest_tree == "Blood" then
			active.role[name] = "TANK"
		elseif highest_tree == "Retribution" or highest_tree == "Arms" or
			highest_tree == "Fury" or highest_tree == "Frost" or
			highest_tree == "Unholy" or highest_tree == "Enhancement" or
			highest_tree == "Elemental" or highest_tree == "Shadow" or 
			highest_tree == "Balance" then
			active.role[name] = "DAMAGER"
		end
		self:Debug( (" %s, %s -- assigned: %s"):format(name, highest_tree, active.role[name] ) )
	
		self:Debug("Sending RosterUpdated message")
		self:SendMessage( "Triage_RosterUpdated" )
	end

	self:Debug( "TalentQuery_Ready finished for " .. name )
end



function Roster:LeftCombat()
	if self.update_in_combat then
		self.update_in_combat = false
		self:Debug("LeftCombat")
		self:Update()
	end
end