--[[ Triage, a Healing assignment manager addon ]]
--[[ tab widths set to 4 for optimal formatting ]]

local _, TriageTable = ...; 

TriageTable.Triage = LibStub("AceAddon-3.0"):NewAddon("Triage",
					"AceConsole-3.0", "AceComm-3.0", "AceEvent-3.0")

local Triage = TriageTable.Triage
_G["Triage"] = Triage

local L = LibStub("AceLocale-3.0"):GetLocale("Triage");


--[[ Addon level variables ]]
Triage.debug = false
Triage.debugFrame = ChatFrame1

Triage.options = {
	type = "group",
	name = L["Options"],
	handler = Triage,
	args = {
		config = {
			type = "execute",
			name = L["Config"],
			desc = L["Show Configuration Dialog"],
			func = function() LibStub("AceConfigDialog-3.0"):Open("Triage") end,
			guiHidden = true,
		},
		channel = {
			name = L["Channel"],
			desc = L["Report results to this channel"],
			type = "input",
			get = "GetReportChannel",
			set = "SetReportChannel",
		},
		debug = {
			name = L["Debugging"],
			desc = L["Toggle Debugging"],
			type = "toggle",
			get = "GetDebugging",
			set = "SetDebugging",
		},
		report = {
			type = "group",
			name = L["Report"],
			handler = Triage,
			desc = L["Report current assignments"],
			args = {
				roster = {
					type = "execute",
					name = L["Report Roster"],
					desc = L["Report current roster"],
					func = function() Triage:Report() end,
					guiHidden = true,
				},
				heal = {
					type = "execute",
					name = L["Report Healers"],
					desc = L["Report current healing assignments"],
					func = function() Triage.Dialog:ReportHealers() end,
					guiHidden = true,
				},
				dps = {
					type = "execute",
					name = L["Report DPS"],
					desc = L["Report current DPS assignments"],
					func = function() Triage.Dialog:ReportDamagers() end,
					guiHidden = true,
				},
				tank = {
					type = "execute",
					name = L["Report Tanks"],
					desc = L["Report current tanking assignments"],
					func = function() Triage.Dialog:ReportTanks() end,
					guiHidden = true,
				},
			},
		},
		dialog = {
			type = "execute",
			name = L["Dialog"],
			desc = L["Show assignment configuration dialog"],
			func = function() Triage.Dialog:ShowDialog() end,
			guiHidden = true,
		},
	},
}



local defaults = {
	profile = {
		channel = nil,
		debug = true,
	}
}



function Triage:GetReportChannel(info)
	return self.db.profile.channel
end

function Triage:SetReportChannel(info, value)
	self.db.profile.channel = value
end

function Triage:GetDebugging(info)
	return self.db.profile.debug
end

function Triage:SetDebugging(info, value)
	self.db.profile.debug = value
	self.debug = value
end



--[[ Debugging functions ]]
function Triage:Debug(str, ...)
	if not self.debug then return end

	if not str or str:len() == 0 then
		str = "nil"
	elseif str:find("%%") and select("#", ...) > 0 then
		str = str:format(...)
	elseif select("#", ...) > 0 then
		str = string.join(" ", str, tostringall(...))
	end

	self.debugFrame:AddMessage("|c933f9fffTriage:|r " .. str)
end



function Triage:Report()
	self:Debug("Reporting")

	if not Triage.Roster then
		self:Debug("No Roster module")
		return
	end

	local raid_size, raid_roster, waiting_roster, class, role =
		Triage.Roster:GetRaidRoster()

	local raid_summary = ("Triage Report -- Raid size: %d"):format(raid_size)
	local raid_list = "Inside:"
	for i = 1, #raid_roster do
		local name = raid_roster[i]
		local short_role = self:GetShortRole( role[name] )
   		raid_list = raid_list.." "..name.."-"..short_role
	end
	local wait_list = "Outside:"
	for i = 1, #waiting_roster do
		local name = waiting_roster[i]
		local short_role = self:GetShortRole( role[name] )
   		wait_list = wait_list.." "..name.."-"..short_role
	end

	self:SendOutput( raid_summary )
	self:SendOutput( raid_list )
	self:SendOutput( wait_list )
end



function Triage:SendOutput( output )
	local channel = Triage.db.profile.channel
	if not channel or channel:trim() == "" then
		self:Debug( output )
		return
	else
		local id, name = GetChannelName( channel )
		if id ~= 0 then
			ChatThrottleLib:SendChatMessage( "NORMAL", "HO",
				output, "CHANNEL", nil, id )
		else
			ChatThrottleLib:SendChatMessage( "NORMAL", "HO", output, channel )
		end
	end
end



function Triage:GetShortRole( role )
	local short_role = "N"
	if role then
		if role == "HEALER" then
			short_role = "H"
		elseif role == "DAMAGER" then
			short_role = "D"
		elseif role == "TANK" then
			short_role = "T"
		end
	end
	return short_role
end



--[[ Primary Addon hooks, initialize, enable, disable ]]
function Triage:OnInitialize()
	-- Code that you want to run when the addon is first loaded goes here.

	self.db = LibStub("AceDB-3.0"):New("TriageDB", defaults, true )

	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")

	self.options.args.profile =
		 LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)

	self.debug = self.db.profile.debug
	self:Debug("OnInitialize")

	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("Triage",
			self.options, {"triage", "tr" } )

	self:RegisterChatCommand("triage", "ProcessCommandLine")
	self:RegisterChatCommand("tr", "ProcessCommandLine")
end



function Triage:ProcessCommandLine(input)

	if not input or input:trim() == "" then
		self:Debug("ProcessCommandLine: Toggle main dialog")
		self.Dialog:Dialog()
	elseif input == "help" or input == "usage" then
		self:Debug("ProcessCommandLine: parsing help")
		LibStub("AceConfigCmd-3.0").HandleCommand(Triage, "triage",
				"Triage", "")
	else
		self:Debug("ProcessCommandLine: parsing commandline")
		LibStub("AceConfigCmd-3.0").HandleCommand(Triage, "triage",
				"Triage", input)
	end
end



function Triage:OnEnable()
	-- Called when the addon is enabled
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
end



function Triage:OnDisable()
	-- Called when the addon is disabled
end



function Triage:RefreshConfig()
	-- Refresh the database profile
	self.debug = self.db.profile.debug
	self:Debug("Loaded Profile", self.db:GetCurrentProfile())
end



--[[ Event callbacks ]]
function Triage:PLAYER_ENTERING_WORLD()
	-- this is needed for zoning while in combat
	self:PLAYER_REGEN_ENABLED()
end



function Triage:PLAYER_REGEN_DISABLED()
	self:Debug("Entering combat")
	self.inCombat = true
end



function Triage:PLAYER_REGEN_ENABLED()
	self.inCombat = InCombatLockdown() == 1
	if not self.inCombat then
		self:Debug("Leaving combat")
		self.Roster:LeftCombat()
	end
end